from PySide2 import QtWidgets, QtCore


class FloatSlider(QtWidgets.QSlider):
    floatValueChanged = QtCore.Signal(float)

    def __init__(self, decimals=3, *args, **kargs):
        super(FloatSlider, self).__init__(*args, **kargs)
        self._multi = 10 ** decimals

        self.valueChanged.connect(self.emitFloatValueChanged)

    def emitFloatValueChanged(self):
        value = float(super(FloatSlider, self).value())/self._multi
        self.floatValueChanged.emit(value)

    def value(self):
        return float(super(FloatSlider, self).value()) / self._multi

    def setMinimum(self, value):
        return super(FloatSlider, self).setMinimum(value * self._multi)

    def setMaximum(self, value):
        return super(FloatSlider, self).setMaximum(value * self._multi)

    def setSingleStep(self, value):
        return super(FloatSlider, self).setSingleStep(value * self._multi)

    def singleStep(self):
        return float(super(FloatSlider, self).singleStep()) / self._multi

    def setValue(self, value):
        super(FloatSlider, self).setValue(int(value * self._multi))


import cv2 as cv
from PySide2 import QtGui
from PySide2.QtGui import QPixmap
from PySide2.QtWidgets import QLabel, QSizePolicy
import qimage2ndarray

import mnd_utils.image
import mnd_utils.misc


class AspectRatioImage(QLabel):

    def __init__(self, ratio=1.0, height_controls_width=True, parent=None):
        super(AspectRatioImage, self).__init__(parent=parent)
        self.ratio = ratio
        self.height_controls_width = height_controls_width
        self.adjusted_to_size = (-1, -1)

        if height_controls_width:
            self.setSizePolicy(QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Ignored))
        else:
            self.setSizePolicy(QSizePolicy(QSizePolicy.Ignored, QSizePolicy.Preferred))

        self.image = None

    def resizeEvent(self, event: QtGui.QResizeEvent):
        event.accept()

        size = event.size()
        if size == self.adjusted_to_size:  # avoid possible infinite recursion loop
            return
        self.adjusted_to_size = size

        width = size.width()
        height = size.height()
        if self.height_controls_width:
            width = round(height * self.ratio)
        else:
            height = round(width * self.ratio)

        self.resize(width, height)

        if self.image is not None:
            self.set_image(self.image)

    def set_image(self, img):
        self.image = img
        size = self.size()
        img = cv.cvtColor(img, cv.COLOR_BGR2RGB)
        img = mnd_utils.image.fit_in_size(img, (size.width(), size.height()))
        image = qimage2ndarray.array2qimage(img)
        self.setPixmap(QPixmap.fromImage(image))


if __name__ == '__main__':
    import sys
    from PySide2 import QtWidgets
    from PySide2.QtWidgets import QWidget, QApplication, QFrame

    app = QApplication(sys.argv)

    window = QWidget()
    window.setMinimumSize(100, 100)

    hlayout = QtWidgets.QHBoxLayout()
    vlayout = QtWidgets.QVBoxLayout()
    label = QLabel('Hello world')
    label.setFrameStyle(QFrame.Box)
    label.setLineWidth(2)
    arimg1 = AspectRatioImage(ratio=1, height_controls_width=True)
    arimg1.set_image(mnd_utils.misc.test_img(1))
    arimg2 = AspectRatioImage(ratio=0.1, height_controls_width=False)
    arimg2.set_image(mnd_utils.misc.test_img(2))
    widget = QWidget()
    widget.setLayout(vlayout)

    hlayout.addWidget(arimg1)
    hlayout.addWidget(label)
    hlayout.addStretch()
    hlayout.addWidget(widget)
    vlayout.addWidget(arimg2)
    window.setLayout(hlayout)

    window.show()

    sys.exit(app.exec_())



import cv2 as cv
import mnd_utils.image
from PySide2 import QtWidgets, QtCore
from PySide2 import QtGui
import qimage2ndarray

from mnd_qtutils.image_edit_layout import ImageEditLayout


class ImageEditWidget(QtWidgets.QWidget):

    def __init__(self, zoom_sensitivity=0.1,  parent=None):
        super(ImageEditWidget, self).__init__(parent=parent)

        self._zoom_sensitivity = zoom_sensitivity
        self._image = None

        self._drag_start_position = None

        self._layout: ImageEditLayout = ImageEditLayout(1, 1)
        self._layout.eventGeometryChanged.connect(self._update_image)

        self._image_label = QtWidgets.QLabel()
        self._layout.addWidget(self._image_label)

        self.setLayout(self._layout)

    def wheelEvent(self, event: QtGui.QWheelEvent):
        angle_delta = event.angleDelta().y()
        degrees = angle_delta / 8
        steps = degrees / 15

        zoom_amount = steps * self._zoom_sensitivity
        self._layout.add_zoom(zoom_amount)

    def mousePressEvent(self, event: QtGui.QMouseEvent):
        if event.button() == QtCore.Qt.MiddleButton:
            self._drag_start_position = event.pos()

    def mouseMoveEvent(self, event: QtGui.QMouseEvent):
        if event.buttons() != QtCore.Qt.MiddleButton:
            return
        pos = event.pos()
        delta = pos - self._drag_start_position
        self._layout.add_center_offset(delta.x(), delta.y())
        self._drag_start_position = pos

    def map_widget_position_to_relative_image_position(self, position: QtCore.QPoint):
        widget_pos = self._image_label.pos()
        widget_size = self._image_label.size()
        relative_x = (position.x() - widget_pos.x()) / widget_size.width()
        relative_y = (position.y() - widget_pos.y()) / widget_size.height()
        return relative_x, relative_y

    def set_image(self, img):
        self._image = img
        h, w = img.shape[:2]
        ratio = w / h
        self._layout.ratio = ratio
        self._update_image()

    def _update_image(self):
        if self._image is None:
            return
        size = self._image_label.size()
        img = cv.cvtColor(self._image, cv.COLOR_BGR2RGB)
        img = mnd_utils.image.fit_in_size(img, (size.width(), size.height()))
        image = qimage2ndarray.array2qimage(img)
        self._image_label.setPixmap(QtGui.QPixmap.fromImage(image))


if __name__ == '__main__':
    import sys
    import mnd_utils.misc

    app = QtWidgets.QApplication(sys.argv)

    window = ImageEditWidget()
    window.setMinimumSize(400, 400)

    img = mnd_utils.misc.test_img()
    window.set_image(img)

    window.show()

    sys.exit(app.exec_())



<?xml version="1.0" encoding="UTF-8"?>
<ui version="4.0">
 <class>Form</class>
 <widget class="QWidget" name="Form">
  <property name="geometry">
   <rect>
    <x>0</x>
    <y>0</y>
    <width>273</width>
    <height>93</height>
   </rect>
  </property>
  <property name="sizePolicy">
   <sizepolicy hsizetype="Preferred" vsizetype="Preferred">
    <horstretch>0</horstretch>
    <verstretch>0</verstretch>
   </sizepolicy>
  </property>
  <property name="windowTitle">
   <string>Form</string>
  </property>
  <layout class="QVBoxLayout" name="verticalLayout">
   <property name="leftMargin">
    <number>0</number>
   </property>
   <property name="topMargin">
    <number>0</number>
   </property>
   <property name="rightMargin">
    <number>0</number>
   </property>
   <property name="bottomMargin">
    <number>0</number>
   </property>
   <item>
    <widget class="QWidget" name="info_layout" native="true">
     <layout class="QHBoxLayout" name="horizontalLayout">
      <item>
       <widget class="QLabel" name="info_label">
        <property name="text">
         <string/>
        </property>
       </widget>
      </item>
      <item>
       <widget class="QLabel" name="percent_label">
        <property name="text">
         <string/>
        </property>
        <property name="alignment">
         <set>Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter</set>
        </property>
       </widget>
      </item>
     </layout>
    </widget>
   </item>
   <item>
    <widget class="QWidget" name="button_widget" native="true">
     <layout class="QHBoxLayout" name="horizontalLayout_2">
      <property name="leftMargin">
       <number>0</number>
      </property>
      <property name="topMargin">
       <number>0</number>
      </property>
      <property name="rightMargin">
       <number>0</number>
      </property>
      <property name="bottomMargin">
       <number>0</number>
      </property>
      <item>
       <spacer name="horizontalSpacer">
        <property name="orientation">
         <enum>Qt::Horizontal</enum>
        </property>
        <property name="sizeHint" stdset="0">
         <size>
          <width>40</width>
          <height>20</height>
         </size>
        </property>
       </spacer>
      </item>
      <item>
       <widget class="QPushButton" name="cancel_button">
        <property name="text">
         <string>Cancelar</string>
        </property>
       </widget>
      </item>
     </layout>
    </widget>
   </item>
  </layout>
 </widget>
 <resources/>
 <connections/>
</ui>



from collections import Generator
import threading

import pypegraph.action


class TaskProgress:

    def __init__(self, progress_value: float = 0, progress_state: str = ""):
        self._progress_value: float = progress_value
        self._progress_state: str = progress_state

    @property
    def progress_value(self):
        return self._progress_value

    @progress_value.setter
    def progress_value(self, value: float):
        self._progress_value = value

    @property
    def progress_state(self):
        return self._progress_state

    @progress_state.setter
    def progress_state(self, state: str):
        self._progress_state = state


class CancellableTask:

    def __init__(self):
        self._running = False
        self._cancel = False
        self._thread = None

        self.event_execution_started = pypegraph.action.Action()
        self.event_execution_finished = pypegraph.action.Action()  # este evento se invoca en el hilo secundario (generalmente)
        self.event_execution_cancelled = pypegraph.action.Action()
        self.event_notify_progress = pypegraph.action.Action()  # este evento se invoca en el hilo secundario (generalmente)

    def is_running(self):
        return self._running

    def execute(self, function, *params, **kparams) -> bool:
        if not self._running:
            self._thread = threading.Thread(target=self._executor, args=(function, *params), kwargs=kparams, daemon=True)
            self._thread.start()
            self.event_execution_started.invoke()
            return True
        return False

    def __call__(self, function, *params, **kparams) -> bool:
        return self.execute(function, *params, **kparams)

    def cancel(self):
        if self._thread and self._running:
            self._cancel = True
            self._thread.join()
            self._thread = None
            self._cancel = False
            self.event_execution_cancelled.invoke()

    def wait_to_finish(self):
        if self._thread and self._running:
            self._thread.join()

    def _executor(self, function, *params, **kparams):
        self._running = True

        res = function(*params, **kparams)
        return_value = None
        if isinstance(res, Generator):
            for yield_value in res:
                return_value = yield_value
                self._handle_yield(yield_value)
                if self._cancel:
                    self._running = False
                    return
        else:
            return_value = res

        self._running = False
        self.event_execution_finished.invoke(return_value)

    def _handle_yield(self, yield_value):
        if yield_value is None or not isinstance(yield_value, TaskProgress):
            return
        self.event_notify_progress.invoke(yield_value)


if __name__ == '__main__':
    def f(N):
        l = [x for x in range(N)]
        for x in l:
            yield x
        print("finished")

    task = CancellableTask()

    task.execute(f, 10000000)
    task.wait_to_finish()
    task.execute(f, N=10000000)
    task.wait_to_finish()



import cv2 as cv
import numpy as np
import qimage2ndarray
from PySide2 import QtWidgets, QtGui, QtCore
from PySide2.QtCore import QFile
from PySide2.QtGui import QIcon, QPixmap
from PySide2.QtUiTools import QUiLoader


def import_from_ui_file(ui_file_path: str):
	ui_file = QFile(ui_file_path)
	ui_file.open(QFile.ReadOnly)
	loader = QUiLoader()
	return loader.load(ui_file)


def setup_widget_from_ui(ui_file_path: str, parent: QtWidgets.QWidget):
	widget = import_from_ui_file(ui_file_path)

	layout = QtWidgets.QVBoxLayout()
	layout.setContentsMargins(0, 0, 0, 0)
	parent.setLayout(layout)
	layout.addWidget(widget)

	return widget


def setup_widget_from_py_ui(pyui_file_path: str, parent: QtWidgets.QWidget):
	widget = import_from_ui_file(pyui_file_path)

	layout = QtWidgets.QVBoxLayout()
	layout.setContentsMargins(0, 0, 0, 0)
	parent.setLayout(layout)
	layout.addWidget(widget)

	return widget


def icon_from_image(img: np.ndarray):
	return QIcon(QPixmap.fromImage(qimage2ndarray.array2qimage(img)))


def remove_widget_from_list_widget(list_widget: QtWidgets.QListWidget, widget: QtWidgets.QWidget):
	items_count = list_widget.count()

	for i in range(items_count):
		item = list_widget.item(i)
		wgt = list_widget.itemWidget(item)
		if wgt == widget:
			list_widget.takeItem(list_widget.row(item))
			break


def get_image_from_drop_event(event: QtGui.QDropEvent):
	mime_data = event.mimeData()
	image = None
	if mime_data.hasUrls():
		urls = mime_data.urls()
		if len(urls) > 1:
			return
		image_path: str = urls[0].url()
		if not image_path.startswith("file:///"):
			return
		# is a file
		image_path = image_path.replace("file:///", '')
		image = cv.imread(image_path)
	elif mime_data.hasFormat('application/x-qabstractitemmodeldatalist'):
		model = QtGui.QStandardItemModel()

		model.dropMimeData(event.mimeData(), QtCore.Qt.CopyAction, 0, 0, QtCore.QModelIndex())
		item = model.item(0, 0)
		image = item.data(QtCore.Qt.UserRole)
	elif mime_data.hasImage():
		print("has image")
	return image


from PySide2 import QtWidgets, QtCore
from typing import Optional


class ImageEditLayout(QtWidgets.QLayout):

    eventGeometryChanged = QtCore.Signal()

    minimum_zoom = 0.05

    def __init__(self, ratio: float, zoom: float = 1, parent: QtWidgets.QWidget = None):
        super(ImageEditLayout, self).__init__(parent=parent)
        self._ratio = ratio
        self._zoom = zoom
        self._center_offset_x = 0
        self._center_offset_y = 0
        self._item: Optional[QtWidgets.QLayoutItem] = None

    @property
    def ratio(self):
        return self._ratio

    @ratio.setter
    def ratio(self, value):
        self._ratio = value
        self.update()

    @property
    def zoom(self):
        return self._zoom

    @zoom.setter
    def zoom(self, value):
        self._zoom = value
        self._zoom = max(self._zoom, self.minimum_zoom)
        self.update()

    def add_zoom(self, amount: float):
        self.zoom += amount

    def add_center_offset(self, offset_x, offset_y):
        self._center_offset_x += offset_x
        self._center_offset_y += offset_y
        self.update()

    def _have_item(self) -> bool:
        return self._item is not None

    def _center_to_topleft(self, container_width, container_height, item_width, item_height, container_x, container_y):
        left = (container_width - item_width) / 2 + container_x
        top = (container_height - item_height) / 2 + container_y
        return left, top

# QLayout overrides ****************************************************************************************************

    def addItem(self, item: QtWidgets.QLayoutItem):
        if not self._have_item():
            self._item = item

    def count(self) -> int:
        count = 1 if self._have_item() else 0
        return count

    def setGeometry(self, geometry: QtCore.QRect):
        super(ImageEditLayout, self).setGeometry(geometry)

        if not self._have_item():
            return

        geom_height = geometry.height()
        geom_width = geometry.width()

        geom_half_width = geom_width / 2
        geom_half_height = geom_height / 2

        item_height = geom_height * self._zoom
        item_width = item_height * self._ratio

        item_half_width = item_width / 2
        item_half_height = item_height / 2

        # clamp offsets
        if item_width > geom_half_width:
            self._center_offset_x = max(self._center_offset_x, -item_half_width)
            self._center_offset_x = min(self._center_offset_x, item_half_width)
        else:
            self._center_offset_x = max(self._center_offset_x, -(geom_half_width - item_half_width))
            self._center_offset_x = min(self._center_offset_x, geom_half_width - item_half_width)

        if item_height > geom_half_height:
            self._center_offset_y = max(self._center_offset_y, -item_half_height)
            self._center_offset_y = min(self._center_offset_y, item_half_height)
        else:
            self._center_offset_y = max(self._center_offset_y, -(geom_half_height - item_half_height))
            self._center_offset_y = min(self._center_offset_y, geom_half_height - item_half_height)

        item_x = geometry.x() + geom_half_width - item_half_width + self._center_offset_x
        item_y = geometry.y() + geom_half_height - item_half_height + self._center_offset_y
        item_height = round(item_height)
        item_width = round(item_width)
        item_x = round(item_x)
        item_y = round(item_y)

        item_geom = QtCore.QRect(0, 0, item_width, item_height)
        self._item.setGeometry(item_geom)
        self._item.widget().move(item_x, item_y)
        self.eventGeometryChanged.emit()

    def sizeHint(self) -> QtCore.QSize:
        size = QtCore.QSize(0, 0)

        if self._have_item():
            item_size_hint = self._item.sizeHint()
            height_hint = round(item_size_hint.height() / self.zoom)
            size.setHeight(height_hint)
            size.setWidth(item_size_hint.width())

        return size

    def minimumSize(self) -> QtCore.QSize:
        size = QtCore.QSize(0, 0)

        if self._have_item():
            size = self._item.minimumSize()

        return size

    def itemAt(self, index: int) -> QtWidgets.QLayoutItem:
        if index < self.count():
            return self._item
        return None

    def takeAt(self, index: int) -> QtWidgets.QLayoutItem:
        item = self._item
        self._item = None
        return item

# **********************************************************************************************************************



import os
from pathlib import Path
from PySide2 import QtWidgets, QtCore

from mnd_qtutils.cancellable_task import CancellableTask, TaskProgress
from mnd_qtutils import qtutils


class ProgressTaskDialog(QtWidgets.QDialog):
    signal_execution_finished = QtCore.Signal()

    def __init__(self, function, params, kparams, cancellable: bool = True, cancellable_task: CancellableTask = None, dialog_title: str = None, parent=None):
        super(ProgressTaskDialog, self).__init__(parent=parent)
        self._cancellable_task = cancellable_task or CancellableTask()
        self._target_function = function
        self._params = params
        self._kparams = kparams
        self._task_result = None
        self._cancelled = False

        self._cancellable_task.event_notify_progress += self._update_task_info
        self._cancellable_task.event_execution_finished += self._on_execution_finished

        self.signal_execution_finished.connect(self._close_dialog)

        self.setWindowTitle(dialog_title or 'Ejecutando...')
        self.setWindowFlags(QtCore.Qt.Window | QtCore.Qt.WindowTitleHint | QtCore.Qt.CustomizeWindowHint)
        self._widget = qtutils.import_from_ui_file(os.path.join(Path(__file__).parent, 'cancellable_task_dialog_ui.py'))

        self._info_label = self._widget.info_label
        self._percent_label = self._widget.percent_label

        self._cancel_button: QtWidgets.QPushButton = self._widget.cancel_button
        if not cancellable:
            self._cancel_button.hide()
        else:
            self._cancel_button.clicked.connect(self._cancel_execution)

        layout = QtWidgets.QVBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)
        layout.addWidget(self._widget)
        self.setLayout(layout)

        self._start_execution()

    @property
    def is_cancelled(self):
        return self._cancelled

    @property
    def task_result(self):
        return self._task_result

    def _start_execution(self):
        if not self._cancellable_task.execute(self._target_function, *self._params, **self._kparams):
            self._close_dialog(False)

    def _on_progress_received(self, progress):
        self.signal_progress_received.emit(progress)

    def _on_execution_finished(self, result):
        self._task_result = result
        self.signal_execution_finished.emit()

    @QtCore.Slot()
    def _close_dialog(self, success: bool = True):
        self._dispose_events()
        if success:
            self.accept()
        else:
            self.reject()
        self.close()

    @QtCore.Slot()
    def _update_task_info(self, progress: TaskProgress):
        if isinstance(progress, TaskProgress):
            text = progress.progress_state
            percent = progress.progress_value
            if text != '':
                self._info_label.setText(text)
            if percent > 0:
                str_percent = f'{"{:.2f}".format(percent)}%'
                self._percent_label.setText(str_percent)
            else:
                self._percent_label.setText('')

    @QtCore.Slot()
    def _cancel_execution(self):
        self._update_task_info(TaskProgress(progress_state='Cancelando...'))
        self._cancellable_task.cancel()
        self._cancelled = True
        self._close_dialog(True)

    def _dispose_events(self):
        self._cancellable_task.event_notify_progress -= self._on_progress_received
        self._cancellable_task.event_execution_cancelled -= self._on_execution_finished

    @staticmethod
    def create_task_dialog(function, *params, cancellable: bool = True, cancellable_task: CancellableTask = None, dialog_title: str = None, **kparams):
        dialog = ProgressTaskDialog(
            function,
            params=params,
            kparams=kparams,
            cancellable=cancellable,
            cancellable_task=cancellable_task,
            dialog_title=dialog_title)
        result = dialog.exec_()
        error = result == QtWidgets.QDialog.Rejected
        return dialog.is_cancelled, error, dialog.task_result


if __name__ == '__main__':
    import sys
    from PySide2.QtWidgets import QApplication

    app = QApplication(sys.argv)

    window = QtWidgets.QWidget()
    window.setMinimumSize(400, 400)
    window.show()

    ct = CancellableTask()
    def f():
        yield TaskProgress(0, 'printiando')
        for i in range(100000):
            print(i)
            yield TaskProgress(i, '')
        print('finished')
    # cancelled, error = CancellableTaskDialog.create_task_dialog(f, ct)
    cancelled, error, result = ProgressTaskDialog.create_task_dialog(f)
    print(cancelled, error)

    sys.exit(app.exec_())
